<?php
/**
 * Statamic
 *
 * @author      Mubashar Iqbal
 * @author      Jack McDade
 * @copyright   2012 Statamic
 * @link        http://www.statamic.com
 * @license     http://www.statamic.com
 *
 */

if ( ! defined('STATAMIC_VERSION')) define('STATAMIC_VERSION', '1.0');

class Statamic {

  public static $publication_states = array('live' => 'Live', 'draft' => 'Draft', 'hidden' => 'Hidden', 'deleted' => 'Deleted');

  public static function load_all_configs() {
    // do all the other files then do the settings
    $pattern = "_config/*.yaml";
    $list = glob($pattern);

    $config = array();
    $file_config = Spyc::YAMLLoad('_config/settings.yaml');
    $config += $file_config;

    foreach ($list as $name) {
      if (!Statamic_Helper::ends_with($name, "_config/settings.yaml")) {
        $file_config = Spyc::YAMLLoad($name);
        $config += $file_config;
      }
    }
    return $config;
  }

  public static function get_site_root() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['_site_root'])) {
      return $app->config['_site_root'];
    }

    return "/";
  }

  public static function get_site_url() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['_site_url'])) {
      return $app->config['_site_url'];
    }

    return "";
  }

  public static function get_site_name() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['_site_name'])) {
      return $app->config['_site_name'];
    }

    return "";
  }

  public static function get_license_key() {
    $app = Slim::getInstance();
    if (isset($app->config['_license_key'])) {
      return $app->config['_license_key'];
    }

    return '';
  }

  public static function get_theme_name() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['_theme'])) {
      return $app->config['_theme'];
    }

    return "default";
  }

  public static function get_theme_type() {
    // default: lex
    $app = Slim::getInstance();
    if (isset($app->config['theme_type'])) {
      return $app->config['theme_type'];
    }

    return "lex";
  }

  public static function get_theme() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['_theme'])) {
      return $app->config['_theme'];
    }

    return "default";
  }

  public static function get_theme_assets_path() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['_theme_assets_path'])) {
      return $app->config['_theme_assets_path'];
    }

    return "";
  }

  public static function get_theme_path() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['theme_path'])) {
      return $app->config['theme_path'];
    }

    return "_themes/".self::get_theme_name()."/";
  }

  public static function get_templates_path() {
    // default: default
    $app = Slim::getInstance();
    if (isset($app->config['templates.path'])) {
      return $app->config['templates.path'];
    }

    return "./_themes/".self::get_theme_name()."/";
  }

  public static function get_content_root() {
    // default: content
    $app = Slim::getInstance();
    if (isset($app->config['_content_root'])) {
      return $app->config['_content_root'];
    }

    return "content";
  }

  public static function get_content_type() {
    $app = Slim::getInstance();

    $content_type = 'md'; # default: markdown
    if (isset($app->config['_content_type']) && $app->config['_content_type'] != 'markdown') {
      $content_type = $app->config['_content_type'];
    }
    return $content_type;
  }

  public static function get_date_format() {
    // default: Ymd
    $app = Slim::getInstance();
    if (isset($app->config['_date_format'])) {
      return $app->config['_date_format'];
    }

    return "Y-m-d";
  }

  public static function get_templates_list() {
    $templates = array();
    $folder = "themes/".self::get_theme_name()."/templates/*";
    $list = glob($folder);
    foreach ($list as $name) {
      $start = strrpos($name, "/")+1;
      $end = strrpos($name, ".");
      $templates[] = substr($name, $start, $end-$start);
    }
    return $templates;
  }

  public static function get_layouts_list() {
    $templates = array();
    $folder = "themes/".self::get_theme_name()."/layouts/*";
    $list = glob($folder);
    foreach ($list as $name) {
      $start = strrpos($name, "/")+1;
      $end = strrpos($name, ".");
      $templates[] = substr($name, $start, $end-$start);
    }
    return $templates;
  }




  public static function get_pagination_variable() {
    $app = Slim::getInstance();

    $var = 'page'; # default: page
    if (isset($app->config['_pagination_variable'])) {
      $var = $app->config['_pagination_variable'];
    }
    return $var;
  }

  public static function get_pagination_style() {
    $app = Slim::getInstance();

    $var = 'prev_next'; # default: prev_next
    if (isset($app->config['_pagination_style'])) {
      $var = $app->config['_pagination_style'];
    }
    return $var;
  }


  public static function is_content_writable() {
    $content_root = self::get_content_root();
    if (is_writable($content_root)) {
      return true;
    }
    return false;
  }

  public static function content_exists($slug, $folder=null) {
    $app = Slim::getInstance();
    $initialize = $app->config;

    $site_root    = self::get_site_root();
    $content_root = self::get_content_root();
    $content_type = self::get_content_type();
    $url_array    = self::get_url_segments();

    $meta_raw = "";
    if ($folder) {
      if (file_exists("{$content_root}/{$folder}/{$slug}.{$content_type}")) { 
        return true;
      }
    } else {
      if (file_exists("{$content_root}/{$slug}.{$content_type}")) { 
        return true;
      }
    }

    return false;
  }

  public static function get_content_meta($slug, $folder=null, $raw=false) {
    $app = Slim::getInstance();
    $initialize = $app->config;

    $site_root    = self::get_site_root();
    $content_root = self::get_content_root();
    $content_type = self::get_content_type();
    $url_array    = self::get_url_segments();

    $meta_raw = "";
    if ($folder) {
      if (file_exists("{$content_root}/{$folder}/{$slug}.{$content_type}")) { 
        $meta_raw = file_get_contents("{$content_root}/{$folder}/{$slug}.{$content_type}");
      }
    } else {
      if (file_exists("{$content_root}/{$slug}.{$content_type}")) { 
        $meta_raw = file_get_contents("{$content_root}/{$slug}.{$content_type}");
      }
    }

    if (Statamic_Helper::ends_with($meta_raw, "---")) {
      $meta_raw .= "\n"; # prevent parse failure
    }
    # Parse YAML Front Matter
    if (stripos($meta_raw, "---") === FALSE) {
      $meta = Spyc::YAMLLoad($meta_raw);
      $meta['content'] = "";
      if ($raw) {
        $meta['content_raw'] = "";
      }
    } else {
      list($yaml, $content) = preg_split("/---/", $meta_raw, 2, PREG_SPLIT_NO_EMPTY);
      $meta = Spyc::YAMLLoad($yaml);
      if ($raw) {
        $meta['content_raw'] = $content;
      }
      $content = self::transform_content($content);

      Lex_Autoloader::register();
      $parser = new Lex_Parser();
      $parser->cumulative_noparse(true);
      $parser->scope_glue(':');
      $c = $parser->parse($content, array_merge($meta, $initialize));
      //$meta['content'] = $content;
      $meta['content'] = $c;
    }

      if (!$raw) {
      $meta['homepage'] = self::get_site_root();
      $meta['raw_url']  = $app->request()->getResourceUri();
      $meta['page_url'] = $app->request()->getResourceUri();

      # Is date formatted correctly?
      if (Statamic_helper::is_date_slug($slug)) {

        $datestamp = Statamic_helper::get_datestamp($slug);
        
        $meta['datestamp'] = $datestamp;
        $meta['date'] = date(self::get_date_format(), $datestamp);
        $meta['page_url'] = substr($meta['page_url'], 11); # clean url override
      } else if (Statamic_helper::is_numeric_slug($slug)) {
        $meta['numeric'] = Statamic_helper::get_numeric($slug);
      }

      # Jam it all together, brother.
      # @todo: functionize/abstract this method for more flexibility and readability
      foreach($meta as $key => $value) {
        if (is_array($value)) {
          $list = array();
          $url_list = array();

          $i = 1;
          $total_results = count($meta[$key]);
          foreach ($meta[$key] as $k => $v) {
            
            if (self::is_taxonomy($v)) {
            
              $url = self::is_taxonomy($v) ? Statamic_helper::reduce_double_slashes(strtolower($site_root.'/'.$folder.'/'.$key.'/'.$v)) : FALSE;

              $list[] = array(
                'name'  => $v,
                'url'   => $url,
                'count' => $i,
                'total_results' => $total_results,
                'first' => $i == 1 ? TRUE : FALSE,
                'last' => $i == $total_results ? TRUE : FALSE
              );
              
              $url_list[] = '<a href="'.$url.'">'.$v.'</a>'; 
            
            } elseif ( ! is_array($v)) {
              
              $list[] = array(
                'name'  => $v,
                'count' => $i,
                'total_results' => $total_results,
                'first' => $i == 1 ? TRUE : FALSE,
                'last' => $i == $total_results ? TRUE : FALSE
              );
            }

            $i++;

          }

          if ( ! is_array($meta[$key])) {
            $meta[$key.'_list'] = implode(', ', $meta[$key]);
          }
          
          if ($url) {
            $meta[$key.'_url_list'] = implode(', ', $url_list);
          }
          if ( ! is_array($meta[$key][0])) {
            $meta[$key] = $list;
          }
        }
      }
    }
    
    if ($raw) {
      return $meta;
    } else {
      return array_merge($initialize, $meta, $url_array);
    }

  }


  public static function get_content_list($folder=null,$limit=null,$offset=0,$future=false,$past=true,$sort_by='date',$sort_dir='desc',$conditions=null,$switch=null,$skip_status=false) {
    $app = Slim::getInstance();

    $content_type = self::get_content_type();

    $list = self::get_content_all($folder, $future, $past, $conditions,$skip_status);

    // default sort is by date
    if ($sort_by == 'title') {
      usort($list, "statamic_sort_by_title");
    } else if ($sort_by == 'random') {
      shuffle($list);
    }

    // default sort is asc
    if ($sort_dir == 'desc') {
      $list = array_reverse($list);
    }

    // handle offset/limit
    if ($offset > 0) {
      $list = array_splice($list, $offset);
    }

    if ($limit) {
      $list = array_splice($list, 0, $limit);
    }

    if ($switch) {
      $switch_vars = explode('|',$switch);
      $switch_count = count($switch_vars);

      $count = 1;
      foreach ($list as $key => $post) {
        $list[$key]['switch'] = $switch_vars[($count -1) % $switch_count];
        $count++;
      }
    }

    return $list;
  }


  public static function get_next_numeric($folder=null) {
    $next = '01';

    $list = self::get_content_all($folder, true, true, null, true);
    if (sizeof($list) > 0) {
      $item = array_pop($list);
      $current = $item['numeric'];
      if ($current <> '') {
        $next = $current + 1;
        $format= '%1$0'.strlen($current).'d';
        $next = sprintf($format, $next);
      }
    }

    return $next;
  }


  public static function get_content_count($folder=null,$future=false,$past=true,$conditions=null) {
    return sizeof(self::get_content_all($folder, $future, $past, $conditions));
  }

  public static function get_content_all($folder=null,$future=false,$past=true,$conditions=null,$skip_status=false) {
    $app = Slim::getInstance();

    $content_type = self::get_content_type();
    $site_root = self::get_site_root();

    $posts = self::get_file_list($folder);
    $list = array();
    $url_array = self::get_url_segments();

    foreach ($posts as $key => $post)
    {
      // starts with numeric value
      unset($list[$key]);
      if (preg_match("/\d[\.\-]/i", $key) && file_exists($post.".{$content_type}")) {
        $list[$key] = Statamic::get_content_meta($key, $folder);

        $data = Statamic::get_content_meta($key, $folder);

        $list[$key]['slug']    = $site_root.'/'.$key;
        $list[$key]['url']     = $folder ? $site_root.$folder."/".$key : $site_root.$key;
        $list[$key]['raw_url'] = $list[$key]['url'];
        
        $date_entry = false;
        if (Statamic_helper::is_date_slug($key)) {
          $datestamp = Statamic_helper::get_datestamp($key);
          $date_entry = true;
          
          # strip the date
          $list[$key]['slug'] = substr($key, 11);
          $list[$key]['url']  = preg_replace("/\d{4}-\d{2}-\d{2}-/", '', $list[$key]['url']); #override
          
          $list[$key]['datestamp'] = $data['datestamp'];
          $list[$key]['date'] = $data['date'];
        } else {
          $list[$key]['slug'] = substr($key, strpos($key, '.')+1);
          $list[$key]['url']  = preg_replace("/\d*\./", '', $list[$key]['url']); #override
        }

        # fully qualified url
        $list[$key]['permalink'] = Statamic_helper::reduce_double_slashes(Statamic::get_site_url().'/'.$list[$key]['url']);

        /* $content  = preg_replace('/<img(.*)src="(.*?)"(.*)\/?>/', '<img \/1 src="'.Statamic::get_asset_path(null).'/\2" /\3 />', $data['content']); */
        $list[$key]['content'] = Statamic::transform_content($data['content']);

        if (!$skip_status) {
          if (isset($data['status']) && $data['status'] != 'live') {
            unset($list[$key]);
          }
        }


        // Remove future entries
        if ($date_entry && $future === FALSE && $datestamp > strtotime(date("Y-m-d"))) {
          unset($list[$key]);
        }

        // Remove past entries
        if ($date_entry && $past === FALSE && $datestamp < strtotime(date("Y-m-d"))) {
          unset($list[$key]);
        }

        if ($conditions) {
          $conditions_array = explode(",", $conditions);
          foreach ($conditions_array as $condition) {
            list($condition_key, $condition_value) = explode(":", $condition);

            $keep = false;
            if (isset($list[$key][$condition_key])) {
              if (is_array($list[$key][$condition_key])) {
                foreach ($list[$key][$condition_key] as $key2 => $value2) {
                  if (strtolower($value2['name']) == strtolower($condition_value)) {
                    $keep = true;
                    break;
                  }
                }
              } else if ($list[$key][$condition_key] == $condition_value)  {
                $keep = true;
              }
            } else {
              $keep = false;
            }
          
            if (!$keep) {
              unset($list[$key]);
            }
          }
        }
      }
    }

    return $list;
  }

  public static function get_content_tree($directory='/',$depth=1,$max_depth=5,$include_pages=false,$include_entries=true) {

    $app = Slim::getInstance();

    $content_root = self::get_content_root();
    $content_type = self::get_content_type();
    $site_root = self::get_site_root();

    $current_url = Statamic_helper::reduce_double_slashes($site_root.'/'.$app->request()->getResourceUri());
    
    $taxonomy_url = FALSE;
    if (self::is_taxonomy_url($current_url)) {
      list($taxonomy_type, $taxonomy_name) = self::get_taxonomy_criteria($current_url);
      $taxonomy_url = self::remove_taxonomy_from_path($current_url, $taxonomy_type, $taxonomy_name);
    }

    $directory = '/'.$directory.'/'; #ensure proper slashing

    $base = ''; 
    if ($directory <> '/') {
      $base = Statamic_helper::reduce_double_slashes("{$content_root}/{$directory}");
    } else if ($directory == '/') {
      $base = "{$content_root}";
    } else {
      $base = "{$content_root}";
    }

    $files = glob("{$base}/*");


    $data = array();
    if ($files) {
      foreach($files as $path) {
        $current_name = basename($path);

        if (!Statamic_helper::starts_with($current_name, '_') && !Statamic_helper::ends_with($current_name, '.yaml')) {
          $node = array();
          $file = substr($path, strlen($base)+1, strlen($path)-strlen($base)-strlen($content_type)-2);
          if (is_dir($path)) {
            $folder = substr($path, strlen($base)+1);
            $node['type']     = 'folder';
            $node['slug']     = basename($folder);
            $node['title']    = ucwords(basename($folder));

            if (Statamic_helper::is_numeric_slug($folder)) {
              $pos = stripos($folder, ".");
              if ($pos !== false) {

                $node['url']      = Statamic_helper::reduce_double_slashes(
                                      Statamic_helper::remove_numerics_from_path(
                                        $site_root.'/'.$directory.'/'.$folder
                                      )
                                    );
                $node['title']    = ucwords(basename(substr($folder, $pos+1)));
              } else {
                $node['title']    = ucwords(basename($folder));
                $node['url']      = Statamic_helper::reduce_double_slashes($site_root.'/'.$directory.'/'.$folder);
              }
            } else {
              $node['title']    = ucwords(basename($folder));
              $node['url']      = Statamic_helper::reduce_double_slashes($site_root.'/'.$directory.'/'.$folder);
            }

            $node['depth']    = $depth;
            $node['children'] = $depth < $max_depth ? self::get_content_tree($directory.$folder.'/', $depth+1, $max_depth, $include_pages, $include_entries) : null;
            $node['is_current']   = $node['url'] == $current_url ? TRUE : FALSE;
          
            $node['is_parent'] = FALSE;
            if ($node['url'] == Statamic_helper::pop_last_segment($current_url) || ($taxonomy_url && $node['url'] == $taxonomy_url)) {
              $node['is_parent'] = TRUE;
            }

            $node['has_children'] = $node['children'] ? TRUE : FALSE;

            // has entries?
            if (file_exists(Statamic_helper::reduce_double_slashes($path."/fields.yaml"))) {
              $node['has_entries'] = TRUE;
            } else {
              $node['has_entries'] = FALSE;
            }

            $meta = self::get_content_meta("page", Statamic_helper::reduce_double_slashes($directory."/".$folder)); 
            if (isset($meta['title'])) $node['title'] = $meta['title'];

            $data[] = $node;
          } else {
              if ($include_pages) {
                if ($file == 'page' || $file == 'feed') {
                  // $node['url'] = $directory;
                  // $node['title'] = basename($directory);

                  // $meta = self::get_content_meta('page', substr($directory, 1));
                  // $node['depth'] = $depth;
                } else {
                  $include = true;

                  if (Statamic_helper::is_date_slug(basename($path))) {
                    if ($include_entries == false) {
                      $include = false;
                    }
                  } else if (Statamic_helper::is_numeric_slug(basename($path))) {
                    if ($include_entries == false) {
                      $include = false;
                    }
                  }

                  if ($include) {
                    $node['type'] = 'file';
                    $node['url'] = Statamic_helper::reduce_double_slashes($directory).basename($path);
                    $node['slug'] = substr(basename($path), 0, -1*(strlen($content_type)+1));;

                    // print "\n B: " . substr(basename($path), 0, -1*(strlen($content_type)+1));
                    // print "\n D: " . $directory;

                    $meta = self::get_content_meta(substr(basename($path), 0, -1*(strlen($content_type)+1)), substr($directory, 1));
                    //$node['meta'] = $meta;

                    if (isset($meta['title'])) $node['title'] = $meta['title'];
                    $node['depth'] = $depth;
                    $data[] = $node;
                  }
                }
              }
          }
        }
      }
    }
    return $data;
  }

  // @todo clean up, build out, figure out what it's doing...
  public static function get_file_list($directory=null) {
    $content_root = self::get_content_root();
    $content_type = self::get_content_type();

    if ($directory) {
      $files = glob("{$content_root}/{$directory}/*.{$content_type}");
    } else {
      $files = glob('{$content_root}/*.{$content_type}');
    }
    $posts = array();

    if ($files) {
      foreach ($files as $file) {
        $len = strlen($content_type);
        $len = $len + 1;
        $len = $len * -1;
        $posts[substr(basename($file), 0, $len)] = substr($file, 0, $len);
      }
    }
    return $posts;
  }

  public static function find_prev($current,$folder=null,$future=false,$past=true) {
    if ($folder == '') {
      $folder = '/';
    }
    $list = Statamic::get_content_list($folder, null, 0, $future, $past);
    $keys = array_keys($list);
    $current_key = array_search($current, $keys);
    if ($current_key !== FALSE) {
      while (key($keys) !== $current_key) next($keys);
        return next($keys);
    }
    return FALSE;
  }

  public static function find_next($current,$folder=null,$future=false,$past=true) {
    if ($folder == '') {
      $folder = '/';
    }
    $list = Statamic::get_content_list($folder, null, 0, $future, $past);
    $keys = array_keys($list);
    $current_key = array_search($current, $keys);
    if ($current_key !== FALSE) {
      while (key($keys) !== $current_key) next($keys);
        return prev($keys); 
    }

    return FALSE;
  }

  public static function get_asset_path($asset) {
    $content_root = self::get_content_root();
    $app = Slim::getInstance();
    return "{$content_root}".$app->request()->getResourceUri().''.$asset;
  }

  public static function transform_content($content) {
    $content_type = self::get_content_type();
    if ($content_type == "markdown" || $content_type == 'md') {
      return Markdown($content);
    } else if ($content_type == 'html') {
      return nl2br($content);
    } else if ($content_type == 'txt') {
      return nl2br(strip_tags($content));
    } else if ($content_type == 'textile') {
      $textile = new Textile();
      return $textile->TextileThis($content);
    }

    return $content;
  }

  public static function get_url_segments()
  {
    $app = Slim::getInstance();
    $segment_array = array('segment_1' => FALSE, 'segment_2' => FALSE, 'segment_3' => FALSE, 'segment_4' => FALSE, 'segment_5' => FALSE, 'segment_6' => FALSE, 'segment_7' => FALSE, 'segment_8' => FALSE, 'segment_9' => FALSE);

    $url = $app->request()->getResourceUri();
    $url_array = explode('/', $url);

    foreach ($url_array as $key => $seg) {
      $segment_array['segment_'.$key++] = $seg;
    }

    return $segment_array;
  }

  public static function is_taxonomy($tax) {
    $app = Slim::getInstance();
    
    if (isset($app->config['_taxonomy'])) {
      $taxonomies = $app->config['_taxonomy'];
      if (in_array($tax, $taxonomies)) {
        return TRUE;
      }
    }

    return TRUE;
  }

  public static function is_taxonomy_url($path) {
    $app = Slim::getInstance();

    if (isset($app->config['_taxonomy'])) {
      $taxonomies = $app->config['_taxonomy'];

      $items = explode("/", $path); # get the last 2 segments of the path, format: /<taxonomy_type>/<slug>
      if (sizeof($items) > 2) {
        $slug = array_pop($items);
        $type = array_pop($items);

        foreach ($taxonomies as $key => $taxonomy) {
          if ($type == $taxonomy) {
            return TRUE;
          }
        }
      }
    }

    return FALSE;
  }

  public static function get_taxonomy_criteria($path) {
    $app = Slim::getInstance();
    if (isset($app->config['_taxonomy'])) {
      $taxonomies = $app->config['_taxonomy'];
      // get the last 2 segments of the path
      // format: /<taxonomy_type>/<slug>
      $items = explode("/", $path);
      if (sizeof($items) > 2) {
        $slug = array_pop($items);
        $type = array_pop($items);

        foreach ($taxonomies as $key => $taxonomy) {
          if ($type == $taxonomy) {
            return array($type, $slug);
          }
        }
      }
    }

    return FALSE;
  }

  public static function remove_taxonomy_from_path($path, $type, $slug) {
    $return = $path;

    $return = substr($path, 0, -1 * strlen("/{$type}/{$slug}"));

    return $return;
  }
}

function statamic_sort_by_title($a, $b) {
  return strcmp($a['title'], $b['title']);
}
